@@ -1,6 +1,6 @@
 # -*- coding: utf-8 -*-
 from psutil import Process, wait_procs, NoSuchProcess
-from robotlibcore import DynamicCore, keyword
+from SeleniumLibrary.base.robotlibcore import DynamicCore, keyword
 from SeleniumTestability.types import ProcessType
 
 
