@@ -17,8 +17,7 @@ log_name = location / "SeleniumTestability.log"
 log_handler = logging.FileHandler(log_name)
 formatter = logging.Formatter("%(asctime)s - %(name)s - %(levelname)s - %(message)s")
 log_handler.setFormatter(formatter)
-LEVELS = {"FAIL": logging.DEBUG, "WARN": logging.WARN, "INFO": logging.INFO,
-          "DEBUG": logging.DEBUG, "TRACE": logging.DEBUG}
+LEVELS = {"FAIL": logging.DEBUG, "WARN": logging.WARN, "INFO": logging.INFO, "DEBUG": logging.DEBUG, "TRACE": logging.DEBUG}
 
 
 @lru_cache(maxsize=1)
