@@ -6,7 +6,17 @@ from os.path import abspath, dirname, join
 from .listener import TestabilityListener
 from .javascript import JS_LOOKUP
 from .logger import get_logger, argstr, kwargstr
-from .types import WebElementType, LocatorType, OptionalBoolType, OptionalStrType, BrowserLogsType, OptionalDictType, is_firefox, StringArray, StorageType
+from .types import (
+    WebElementType,
+    LocatorType,
+    OptionalBoolType,
+    OptionalStrType,
+    BrowserLogsType,
+    OptionalDictType,
+    is_firefox,
+    StringArray,
+    StorageType,
+)
 from robot.utils import is_truthy, timestr_to_secs, secs_to_timestr
 from selenium.webdriver.support.ui import WebDriverWait
 from selenium.common.exceptions import TimeoutException, WebDriverException
@@ -197,16 +207,46 @@ class SeleniumTestability(LibraryComponent):
         self.browser_warn_shown = False
         self.empty_log_warn_shown = False
         self.ff_log_pos = {}  # type: Dict[str, int]
+        self.testability_config = None  # type: OptionalDictType
 
     @log_wrapper
     def _inject_testability(self: "SeleniumTestability") -> None:
         """
         Injects SeleniumTestability javascript bindings into a current browser's current window. This should happen automatically vie SeleniumTestability's internal `Event Firing Webdriver` support but keyword is provided also.
         """
+
+        if self.testability_config:
+            self.ctx.driver.execute_script(JS_LOOKUP["testability_config"], self.testability_config)
+
         with open(self.js_bundle, "r") as f:
             buf = f.read()
             self.ctx.driver.execute_script("{};".format(buf))
 
+    @log_wrapper
+    @keyword
+    def set_testability_config(self: "SeleniumTestability", config: Dict) -> None:
+        """
+        Sets configuration that is used by SUT.  `config` dictionary should can have following keys: `maxTimeout` and `blacklist`.
+
+        Configuration has to be set before opening any browsers or before SUT is instrumented either automatically or manually.
+
+        - `maxTimeout` affects affects javascript `setTimeout()` calls and what is the maximum timeout that should be observed and waited. Value is milliseconds and defaults to 5000
+        - `blacklist` is array of dictionaries where each array element have 2 fields. `url` and `method`.  `url` should be a regular expression that matches the actual url or url of the Request object's url field.
+        Do note that the regular expression should match what is actually passed to the async method - not the fully qualied url.
+
+        Parameters:
+        - ``config`` dictionary of testability.js config options.
+
+        Example:
+
+        | &{longfetch}=           | `Create Dictionary`   | url=.*longfetch.*            |  method=GET              |
+        | @{blacklist}=           | `Create List`         | ${longfetch}                 |                          |
+        | ${tc}=                  | `Create Dictionary`   | maxTimeout=5000              |  blacklist=${blacklist}  |
+        | Set Testability Config  |  ${tc}                |                              |                          |
+.
+        """
+        self.testability_config = config
+
     @log_wrapper
     @keyword
     def instrument_browser(self: "SeleniumTestability") -> None:
@@ -382,7 +422,7 @@ class SeleniumTestability(LibraryComponent):
         Converts a cookie string into python dict.
         """
         ret = {}
-        cookie = SimpleCookie()
+        cookie = SimpleCookie()  # type: SimpleCookie
         cookie.load(cookies)
         for key, morsel in cookie.items():
             ret[key] = morsel.value
@@ -486,19 +526,27 @@ class SeleniumTestability(LibraryComponent):
 
     @log_wrapper
     @keyword
-    def scroll_to_bottom(self: "SeleniumTestability") -> None:
+    def scroll_to_bottom(self: "SeleniumTestability", smooth: bool = False) -> None:
         """
         Scrolls current window to the bottom of the page
+        Parameters:
+         - ``smooth`` if sets to true, enables smooth scrolling, otherwise instant.
         """
-        self.ctx.driver.execute_script(JS_LOOKUP["scroll_to_bottom"])
+        smooth = bool(smooth)
+        behavior = "smooth" if smooth else "instant"
+        self.ctx.driver.execute_script(JS_LOOKUP["scroll_to_bottom"], behavior)
 
     @log_wrapper
     @keyword
-    def scroll_to_top(self: "SeleniumTestability") -> None:
+    def scroll_to_top(self: "SeleniumTestability", smooth: bool = False) -> None:
         """
-        Scrolls current window to the bottom of the page
+        Scrolls current window to the top of the page
+        Parameters:
+         - ``smooth`` if sets to true, enables smooth scrolling, otherwise instant.
         """
-        self.ctx.driver.execute_script(JS_LOOKUP["scroll_to_top"])
+        smooth = bool(smooth)
+        behavior = "smooth" if smooth else "instant"
+        self.ctx.driver.execute_script(JS_LOOKUP["scroll_to_top"], behavior)
 
     @log_wrapper
     @keyword
@@ -597,7 +645,9 @@ class SeleniumTestability(LibraryComponent):
             raise AssertionError("Element with locator {} is blocked".format(locator))
 
     def _get_ff_log(self: "SeleniumTestability", name: str) -> BrowserLogsType:
-        matcher = r"^(?P<source>JavaScript|console)(\s|\.)(?P<level>warn.*?|debug|trace|log|error|info):\s(?P<message>(?!resource:).*)$"
+        matcher = (
+            r"^(?P<source>JavaScript|console)(\s|\.)(?P<level>warn.*?|debug|trace|log|error|info):\s(?P<message>(?!resource:).*)$"
+        )
         LEVEL_LOOKUP = {
             "log": "INFO",
             "warn": "WARN",
@@ -637,6 +687,8 @@ class SeleniumTestability(LibraryComponent):
 
         Note: On firefox, the firefox profile has to have `devtools.console.stdout.content` property to be set.
         This can be done automatically with `Generate Firefox Profile` and then pass that to `Open Browser`.
+
+        This keyword will mostly likely not work with remote seleniun driver!
         """
         ret = []  # type: BrowserLogsType
         try:
@@ -744,19 +796,27 @@ class SeleniumTestability(LibraryComponent):
     @keyword
     def generate_firefox_profile(
         self: "SeleniumTestability",
-        options: OptionalDictType = None,
+        preferences: OptionalDictType = None,
         accept_untrusted_certs: bool = False,
         proxy: OptionalStrType = None,
     ) -> FirefoxProfile:
+        """
+        Generates a firefox profile that sets up few required preferences for SeleniumTestability to support all necessary features.
+        Parameters:
+        - ``preferences`` - firefox profile preferences in dictionary format.
+        - ``accept_untrusted_certs`` should we accept untrusted/self-signed certificates.
+        - ``proxy`` proxy options
+
+        Note: If you opt out using this keyword, you are not able to get logs with ``Get Logs`` and Firefox.
+        """
         profile = FirefoxProfile()
-        if options:
-            for key, value in options.items():  # type: ignore
+        if preferences:
+            for key, value in preferences.items():  # type: ignore
                 profile.set_preference(key, value)
 
         profile.set_preference("devtools.console.stdout.content", True)
 
-        if accept_untrusted_certs:
-            profile.accept_untrusted_certs
+        profile.accept_untrusted_certs = accept_untrusted_certs
 
         if proxy:
             profile.set_proxy(proxy)
@@ -767,19 +827,34 @@ class SeleniumTestability(LibraryComponent):
     @log_wrapper
     @keyword
     def get_storage_length(self: "SeleniumTestability", storage_type: str = "localStorage") -> int:
+        """
+        Returns a length (# of items) in specified storage.
+        Parameters:
+        - ``storage_type`` name of the storage. Valid options: localStorage, sessionStorage
+        """
         return self.ctx.driver.execute_script(JS_LOOKUP["storage_length"], storage_type)
 
     @log_wrapper
     @keyword
     def get_storage_keys(self: "SeleniumTestability", storage_type: str = "localStorage") -> StringArray:
+        """
+        Returns a list of keys in specified storage.
+        Parameters:
+        - ``storage_type`` name of the storage. Valid options: localStorage, sessionStorage
+        """
         return self.ctx.driver.execute_script(JS_LOOKUP["storage_keys"], storage_type)
 
     @log_wrapper
     @keyword
     def get_storage_item(self: "SeleniumTestability", key: str, storage_type: str = "localStorage") -> StorageType:
+        """
+        Returns value of ``key`` from specified storage.
+        Parameters:
+        - ``key`` name of the storage key
+        - ``storage_type`` name of the storage. Valid options: localStorage, sessionStorage
+        """
         matcher = r"^{.*}$"
         storage_item = self.ctx.driver.execute_script(JS_LOOKUP["storage_getitem"], storage_type, key)
-        print(f"TYPE OF {key} is {type(storage_item)}")
         if isinstance(storage_item, str) and re.match(matcher, storage_item):
             storage_item = json.loads(storage_item)
         return storage_item
@@ -787,6 +862,13 @@ class SeleniumTestability(LibraryComponent):
     @log_wrapper
     @keyword
     def set_storage_item(self: "SeleniumTestability", key: str, value: StorageType, storage_type: str = "localStorage") -> None:
+        """
+        Sets a value to the key in specified storage_type
+        Parameters:
+        - ``key`` name of the key
+        - ``value`` value that should be set to key
+        - ``storage_type`` name of the storage. Valid options: localStorage, sessionStorage
+        """
         if isinstance(value, dict):
             value = json.dumps(value)
         self.ctx.driver.execute_script(JS_LOOKUP["storage_setitem"], storage_type, key, value)
@@ -794,9 +876,20 @@ class SeleniumTestability(LibraryComponent):
     @log_wrapper
     @keyword
     def clear_storage(self: "SeleniumTestability", storage_type: str = "localStorage") -> None:
+        """
+        Clears all values in specified storage.
+        Parameters:
+        - ``storage_type`` name of the storage. Valid options: localStorage, sessionStorage
+        """
         self.ctx.driver.execute_script(JS_LOOKUP["storage_clear"], storage_type)
 
     @log_wrapper
     @keyword
     def remove_storage_item(self: "SeleniumTestability", key: str, storage_type: str = "localStorage") -> None:
+        """
+        Removes a key and its value from specified storage
+        Parameters:
+        - ``key`` name of the key
+        - ``storage_type`` name of the storage. Valid options: localStorage, sessionStorage
+        """
         return self.ctx.driver.execute_script(JS_LOOKUP["storage_removeitem"], storage_type, key)
